
/*
 __  __           _           _       
|  \/  | ___  ___| |__  _   _| | __ _ 
| |\/| |/ _ \/ __| '_ \| | | | |/ _` |
| |  | |  __/\__ \ | | | |_| | | (_| |
|_|  |_|\___||___/_| |_|\__,_|_|\__,_|
 _____         _   ____                                  
|_   _|____  _| |_/ ___|  ___ __ _ _ __  _ __   ___ _ __ 
  | |/ _ \ \/ / __\___ \ / __/ _` | '_ \| '_ \ / _ \ '__|
  | |  __/>  <| |_ ___) | (_| (_| | | | | | | |  __/ |   
  |_|\___/_/\_\\__|____/ \___\__,_|_| |_|_| |_|\___|_|   
                                                         
This parser was written in the distant past by Nick Porcino for the public domain
and is freely available on an as is basis. It is meant for educational
purposes and is not suitable for any particular purpose. No warranty is expressed
or implied. Use at your own risk. Do not operate heavy machinery or governments
while using this code.

The types referenced are compatible with the C99 stdint.h types.
*/

#pragma once

#ifndef TEXTSCANNER_H
#define TEXTSCANNER_H

#include <stdint.h>

namespace TextScanner
{
	// Get Token
	char const* GetToken						(char delim, char const* pCurr, char const* pEnd, char const*& resultStringBegin, uint32_t& stringLength);
	char const* GetTokenWSDelimited				(char const* pCurr, char const* pEnd, char const*& resultStringBegin, uint32_t& stringLength);
	char const* GetTokenAlphaNumeric			(char const* pCurr, char const* pEnd, char const*& resultStringBegin, uint32_t& stringLength);
	char const* GetNameSpacedTokenAlphaNumeric	(char const* pCurr, char const* pEnd, char const*& resultStringBegin, uint32_t& stringLength);

	// Get Value
	char const* GetString	(char const* pCurr, char const* pEnd, char const*& resultStringBegin, uint32_t& stringLength);
	char const* GetInt		(char const* pCurr, char const* pEnd, int16_t& result);
	char const* GetInt		(char const* pCurr, char const* pEnd, int32_t& result);
	char const* GetInt		(char const* pCurr, char const* pEnd, uint32_t& result);
	char const* GetHex		(char const* pCurr, char const* pEnd, uint32_t& result);
	char const* GetFloat	(char const* pcurr, char const* pEnd, float& result);

	// Scan
	char const* ScanForCharacter			(char delim, char const* pCurr, char const* pEnd);
	char const* ScanBackwardsForCharacter	(char delim, char const* pCurr, char const* pEnd);
	char const* ScanForWhiteSpace			(char const* pCurr, char const* pEnd);
	char const* ScanBackwardsForWhiteSpace	(char const* pCurr, char const* pStart);
	char const* ScanForNonWhiteSpace		(char const* pCurr, char const* pEnd);
	char const* ScanForTrailingNonWhiteSpace(char const* pCurr, char const* pEnd);
	char const* ScanForQuote				(char const* pCurr, char const* pEnd);
	char const* ScanForEndOfLine			(char const* pCurr, char const* pEnd);
	char const* ScanForLastCharacterOnLine	(char const* pCurr, char const* pEnd);
	char const* ScanForBeginningOfNextLine	(char const* pCurr, char const* pEnd);
	char const* ScanPastCPPComments			(char const* pCurr, char const* pEnd);

	// Simple Tests
	template <class Type> inline bool IsWhiteSpace(Type test)	{ return (test == 9 || test == ' ' || test == 13 || test == 10);	}
	template <class Type> inline bool IsNumeric(Type test)		{ return (test >= '0' && test <= '9');	}
	template <class Type> inline bool IsAlpha(Type test)		{ return ((test >= 'a' && test <= 'z') || (test >= 'A' && test <= 'Z'));	}
}

#endif
